/**
 * EasyBeans
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ENCBindingHolder.java 4508 2009-02-03 16:18:05Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.common.impl.enc;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.ee.metadata.common.api.enc.IENCBinding;
import org.ow2.util.ee.metadata.common.api.enc.IENCBindingHolder;
import org.ow2.util.ee.metadata.common.api.enc.IInjectionTargetEnc;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;

/**
 * Class that can manage different kind of binding used in ENC.
 * @author Florent BENOIT
 */
public class ENCBindingHolder implements IENCBindingHolder {

    /**
     * List of field injection targets for each class.
     */
    private Map<String, List<IInjectionTargetEnc>> fieldInjections = null;

    /**
     * List of method injection targets for each class.
     */
    private Map<String, List<IInjectionTargetEnc>> methodInjections = null;

    /**
     * List of binding for Persistence context (Entity Manager).
     */
    private List<IENCBinding<IJavaxPersistenceContext>> persistenceContextBindings = null;

    /**
     * List of binding for Persistence unit (Entity Manager Factory).
     */
    private List<IENCBinding<IJavaxPersistenceUnit>> persistenceUnitBindings = null;

    /**
     * List of binding for javax.Annotation.Resources.
     */
    private Map<String, IENCBinding<IJAnnotationResource>> resourceBindings = null;

    /**
     * List of binding for javax.ejb.EJB(s).
     */
    private List<IENCBinding<IJEjbEJB>> ejbBindings = null;

    /**
     * List of binding for javax.ws.jaxws.WebServiceRef(s).
     */
    private List<IENCBinding<IJaxwsWebServiceRef>> wsBindings = null;


    /**
     * Default constructor.
     */
    public ENCBindingHolder() {
        this.persistenceContextBindings = new ArrayList<IENCBinding<IJavaxPersistenceContext>>();
        this.persistenceUnitBindings = new ArrayList<IENCBinding<IJavaxPersistenceUnit>>();
        this.resourceBindings = new HashMap<String, IENCBinding<IJAnnotationResource>>();
        this.ejbBindings = new ArrayList<IENCBinding<IJEjbEJB>>();
        this.wsBindings = new ArrayList<IENCBinding<IJaxwsWebServiceRef>>();
        this.fieldInjections = new HashMap<String, List<IInjectionTargetEnc>>();
        this.methodInjections = new HashMap<String, List<IInjectionTargetEnc>>();
    }

    /**
     * Add a given persistence context binding.
     * @param binding the given binding.
     */
    public void addPersistenceContextBinding(final IENCBinding<IJavaxPersistenceContext> binding) {
        this.persistenceContextBindings.add(binding);
    }

    /**
     * @return the list of persistence context bindings.
     */
    public List<IENCBinding<IJavaxPersistenceContext>> getPersistenceContextBindings() {
        return persistenceContextBindings;
    }

    /**
     * Add a given persistence unit binding.
     * @param binding the given binding.
     */
    public void addPersistenceUnitBinding(final IENCBinding<IJavaxPersistenceUnit> binding) {
        this.persistenceUnitBindings.add(binding);
    }

    /**
     * @return the list of persistence unit bindings.
     */
    public List<IENCBinding<IJavaxPersistenceUnit>> getPersistenceUnitBindings() {
        return persistenceUnitBindings;
    }

    /**
     * Add a given resource binding.
     * @param binding the given binding.
     */
    public void addResourceBinding(final IENCBinding<IJAnnotationResource> binding) {
        // Check already exists ?
        String bindingName = binding.getName();

        IENCBinding<IJAnnotationResource> existingResource = resourceBindings.get(bindingName);
        // Already defined, ignore it
        if (existingResource != null) {
            return;
        }

        this.resourceBindings.put(bindingName, binding);
    }
    /**
     * @return the list of resource bindings.
     */
    public List<IENCBinding<IJAnnotationResource>> getResourceBindings() {
        Collection<IENCBinding<IJAnnotationResource>> collection = resourceBindings.values();
        List<IENCBinding<IJAnnotationResource>> list = new ArrayList<IENCBinding<IJAnnotationResource>>();
        if (collection != null) {
            for (IENCBinding<IJAnnotationResource> item : collection) {
                list.add(item);
            }
        }
        return list;
    }

    /**
     * Add a given EJB binding.
     * @param binding the given binding.
     */
    public void addEJBBinding(final IENCBinding<IJEjbEJB> binding) {
        this.ejbBindings.add(binding);
    }
    /**
     * @return the list of EJB bindings.
     */
    public List<IENCBinding<IJEjbEJB>> getEJBBindings() {
        return ejbBindings;
    }

    /**
     * Add a given WS binding.
     * @param binding the given binding.
     */
    public void addWebServicesBinding(final IENCBinding<IJaxwsWebServiceRef> binding) {
        this.wsBindings.add(binding);
    }
    /**
     * @return the list of WS bindings.
     */
    public List<IENCBinding<IJaxwsWebServiceRef>> getWebServicesBindings() {
        return wsBindings;
    }

    /**
     * Add a given field injection target for a given classname
     * @param injectionTargetEnc the given field injection target.
     */
    public void addFieldInjectionTarget(final String className, final IInjectionTargetEnc injectionTargetEnc) {
        List<IInjectionTargetEnc> fieldInjectionTargets = fieldInjections.get(className);
        // Init list if not already done
        if (fieldInjectionTargets == null) {
            fieldInjectionTargets = new ArrayList<IInjectionTargetEnc>();
            fieldInjections.put(className, fieldInjectionTargets);
        }

        // add in the list
        fieldInjectionTargets.add(injectionTargetEnc);
    }

    /**
     * Add a given method injection target for a given classname
     * @param injectionTargetEnc the given method injection target.
     */
    public void addMethodInjectionTarget(final String className, final IInjectionTargetEnc injectionTargetEnc) {
        List<IInjectionTargetEnc> methodInjectionTargets = methodInjections.get(className);
        // Init list if not already done
        if (methodInjectionTargets == null) {
            methodInjectionTargets = new ArrayList<IInjectionTargetEnc>();
            methodInjections.put(className, methodInjectionTargets);
        }

        // add in the list
        methodInjectionTargets.add(injectionTargetEnc);
    }


    /**
     * @param className the name of the class for which the fields are requested
     * @return the injection target enc objects for a given classname
     */
    public List<IInjectionTargetEnc> getFieldInjectionTargets(final String className) {
        return fieldInjections.get(className);
    }

    /**
     * @param className the name of the class for which the methods are requested
     * @return the injection target enc objects for a given classname
     */
    public List<IInjectionTargetEnc> getMethodInjectionTargets(final String className) {
        return methodInjections.get(className);
    }

}
