/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: HandlerChainsLoader.java 4689 2009-02-23 14:23:05Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.common.impl.xml.parsing;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.util.Collection;
import java.util.Collections;

import org.ow2.util.ee.metadata.common.api.xml.struct.IHandlerChains;
import org.ow2.util.ee.metadata.common.impl.xml.struct.HandlerChains;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.ow2.util.xml.SchemaEntityResolver;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

/**
 * The HandlerChainsLoader is responsible to load a handler-chains XML file.
 *
 * @author Guillaume Sauthier
 */
public class HandlerChainsLoader extends CommonDeploymentDescLoader {

    /**
     * Logger.
     */
    private static final Log logger = LogFactory.getLog(HandlerChainsLoader.class);

    /**
     * All schemas used by HandlerChains.
     */
    private static final String[] HANDLER_CHAINS_SCHEMAS = new String[]{
            IEntityResolver.PREFIX + IEntityResolver.JAVAEE5_SCHEMA,
            IEntityResolver.PREFIX + IEntityResolver.JAVAEE_WEBSERVICES_CLIENT_SCHEMA,
            IEntityResolver.PREFIX + IEntityResolver.JAVAEE_HANDLER_CHAINS_SCHEMA,
            IEntityResolver.PREFIX + IEntityResolver.XML_SCHEMA};

    /**
     * Loaded HandlerChains.
     */
    private HandlerChains handlerChains;

    /**
     * XML handler-chains element.
     */
    private Element handlerChainsElement;


    /**
     * Load an XML accessible from the given URL
     * @param url handler-chains.xml URL
     */
    private HandlerChainsLoader(URL url) {
        super(url);
        handlerChains = new HandlerChains();
    }

    /**
     * Load an XML Element
     * @param element handler-chains root element
     */
    private HandlerChainsLoader(Element element) {
        // We're not parsing an XML stream here
        super(null);
        this.handlerChainsElement = element;
        handlerChains = new HandlerChains();
    }

    /**
     * Load the handler-chains.xml file.
     *
     * @param url the URL of the XML file.
     * @param validation true for forcing validation
     * @return an application object.
     * @throws ParsingException if parsing of XML file fails.
     */
    public static IHandlerChains loadHandlerChains(final URL url, boolean validation) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        HandlerChainsLoader handlerChainsLoader = new HandlerChainsLoader(url);
        handlerChainsLoader.setValidating(validation);
        handlerChainsLoader.parse();
        return handlerChainsLoader.getHandlerChains();
    }

    /**
     * Load the handler-chains Element.
     *
     * @param handlerChainsElement main element to be parsed
     * @return an application object.
     * @throws ParsingException if parsing of XML file fails (should never
     *         happen since we simply convert an already parsed XML element
     *         into an Object structure).
     */
    public static IHandlerChains loadHandlerChains(final Element handlerChainsElement) throws ParsingException {
        logger.debug("Analyzing element {0}", handlerChainsElement);

        HandlerChainsLoader handlerChainsLoader = new HandlerChainsLoader(handlerChainsElement);
        handlerChainsLoader.parse();
        return handlerChainsLoader.getHandlerChains();
    }

    /**
     * Analyze the URL and build a HandlerChains object which contains data of the XML.
     *
     * @throws ParsingException if the analyze of the file fails.
     */
    private void parse() throws ParsingException {

        if (handlerChainsElement == null) {
            // Get document
            Document document = loadDocument();

            // Root element = <handler-chains>
            handlerChainsElement = document.getDocumentElement();
        }

        if (handlerChainsElement.getNamespaceURI() != null) {
            // Define NS
            defineXMLNS(handlerChainsElement);
        }

        analyzeHandlerChain(handlerChainsElement, handlerChains);

    }

    /**
     * Load the handler-chains.xml document
     * @return the XML Document element
     * @throws ParsingException if something went wrong during parsing
     */
    private Document loadDocument() throws ParsingException {
        Document document;
        Reader reader = null;
        try {
            String handlerChainsSchemaLocation = "http://java.sun.com/xml/ns/javaee "
                                               + "http://java.sun.com/xml/ns/javaee/javaee_web_services_metadata_handler_2_0.xsd";
            Collection<String> schemaLocations = Collections.singletonList(handlerChainsSchemaLocation);

            // Wraps the InputStream in an InputSource
            reader = new InputStreamReader(getURL().openStream());
            InputSource inputSource = new InputSource(reader);

            document = DocumentParser.getDocument(inputSource,
                                                  isValidating(),
                                                  new HandlerChainsEntityResolver(),
                                                  schemaLocations);
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url " + getURL(), e);
        } catch (IOException e) {
            throw new ParsingException("Cannot open the url " + getURL(), e);
        } finally {
            // close InputStreamReader when parsing is finished
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    // Ignore
                }
            }
        }
        return document;
    }

    /**
     * @return the parsed handler-chains parsed element.
     */
    private IHandlerChains getHandlerChains() {
        return handlerChains;
    }


    /**
     * Entity resolver allowing to find schema within the classloader.
     *
     * @author Guillaume Sauthier
     */
    private class HandlerChainsEntityResolver extends SchemaEntityResolver implements IEntityResolver {

        /**
         * Constructor. Finds the XSD with classloader.
         */
        public HandlerChainsEntityResolver() {
            super(HANDLER_CHAINS_SCHEMAS);
        }

    }
}
